﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace TabControl
{
    [TemplateVisualState(GroupName = "EditStates", Name = "EditMode")]
    [TemplateVisualState(GroupName = "EditStates", Name = "ViewMode")]
    public class EditableTabHeader : ContentControl
    {
        private TextBox textBox;
        private DateTime previosLeftClickTime = DateTime.Now;
        private Point previosLeftClickPoint;
        private TimeSpan doubleClickSpan = TimeSpan.FromSeconds(0.4);
        public static readonly DependencyProperty IsInEditModeProperty = DependencyProperty.Register(
            "IsInEditMode",
            typeof(bool),
            typeof(EditableTabHeader),
            new PropertyMetadata(OnIsInEditModeChanged));

        public EditableTabHeader()
        {
            DefaultStyleKey = typeof(EditableTabHeader);
        }
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            this.textBox = this.GetTemplateChild("TextBox") as TextBox;
            this.textBox.LostFocus += new RoutedEventHandler(textBox_LostFocus);
            this.textBox.GotFocus += new RoutedEventHandler(text_Box_GotFocus);
        }
        private void text_Box_GotFocus(object sender, RoutedEventArgs e)
        {
           
        }
        private void textBox_LostFocus(object sender, RoutedEventArgs e)
        {
            this.IsInEditMode = false;
        }
        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);
            var currentTime = DateTime.Now;
            var currentPoint = e.GetPosition(this);
            var durationBetweenClicks = currentTime - previosLeftClickTime;
            if (currentPoint == previosLeftClickPoint && durationBetweenClicks < this.doubleClickSpan)
            {
                e.Handled = true;
                this.IsInEditMode = !this.IsInEditMode;
            }
            this.previosLeftClickTime = DateTime.Now;
            this.previosLeftClickPoint = e.GetPosition(this);
        }
        public bool IsInEditMode
        {
            get { return (bool)GetValue(IsInEditModeProperty); }
            set { SetValue(IsInEditModeProperty, value); }
        }
        private static void OnIsInEditModeChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var editableContentControl = sender as EditableTabHeader;
            var newValue = (bool)e.NewValue;
            if (!newValue)
            {
                editableContentControl.Content = editableContentControl.textBox.Text;
            }
            editableContentControl.ChangeVisualStates();
        }
        public void ChangeVisualStates()
        {
            if (this.IsInEditMode)
            {
                VisualStateManager.GoToState(this, "EditMode", true);
            }
            else
            {
                VisualStateManager.GoToState(this, "ViewMode", true);
            }
        }
    }
}
